# OBJ.BOTS.DCC - timestamped JSON objects
#
# obj/bots/dcc.py (direct client to client bot)
#
# Copyright 2017,2018 B.H.J Thate
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice don't have to be included.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# 24-09-2018 As the creator of this file, I disclaim all rights on  this file. 
#
# Bart Thate
# Heerhugowaard
# The Netherlands

""" direct client to client bot. """

__version__ = 1

## imports

from obj.bots.base import Bot, Users
from obj.bots.main import cfg
from obj.event import Event

import io
import logging
import os
import re
import socket
import sys

## classes

class Event(Event):

    """ DCC event. """

    pass

class DCC(Bot):

    def announce(self, txt):
        """ announce on socket. """
        self.raw(txt)        

    def connect(self, event):
        users = Users()
        if not users.allowed(event.origin, "DCC"):
            return
        arguments = event.txt.split()
        addr = arguments[3]
        port = arguments[4][:-1]
        port = int(port)
        if re.search(':', addr):
            s = socket.socket(socket.AF_INET6, socket.SOCK_STREAM)
        else:
            s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.connect((addr, port))
        s.send(bytes('Welcome to %s %s !!\n' % (cfg.name, event.nick), "utf-8"))
        s.setblocking(True)
        os.set_inheritable(s.fileno(), os.O_RDWR)
        self.start(s, event.origin)

    def errored(self, event):
        logging.error(event.trace)

    def event(self, txt="", origin="", options=""):
        e = Event()
        e._sock = self._sock
        e._sockfile = self._sockfile
        e.orig = repr(self)
        e.channel = self.origin
        e.origin = origin or "root@dcc"
        e.txt = txt
        e.options = options
        return e
        
    def get_event(self):
        txt = self._sockfile.readline()
        return self.event(txt)

    def raw(self, txt):
        try:
            self._sockfile.write(txt)
            self._sockfile.write("\n")
            self._sockfile.flush()
        except:
            pass

    def say(self, channel, txt):
        self.raw(txt)

    def start(self, sock, origin, **kwargs):
        self.origin = origin
        self._sock = sock
        self._sockfile = self._sock.makefile("rw")
        super().start()
